package testcase;

import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;

import org.teiid.language.QueryExpression;
import org.teiid.metadata.Column.SearchType;
import org.teiid.metadata.MetadataFactory;
import org.teiid.metadata.Procedure;
import org.teiid.metadata.RuntimeMetadata;
import org.teiid.metadata.Table;
import org.teiid.translator.ExecutionContext;
import org.teiid.translator.ExecutionFactory;
import org.teiid.translator.ResultSetExecution;
import org.teiid.translator.Translator;
import org.teiid.translator.TranslatorException;
import org.teiid.translator.TypeFacility;

@Translator(name = "timerange-async", description = "A translator to generate a timepoints with continuous execution")
public class TimerangeExecutionFactory
    extends ExecutionFactory<Object, Object>
{

    public static final String             EXECUTION_COUNT = "EXECUTION_COUNT";

    public static final String             ACTUAL_PERIOD   = "ACTUAL_PERIOD";

    public static final String             START           = "TS_START";

    public static final String             STOP            = "TS_STOP";

    public static final String             PERIOD          = "PERIOD";

    public static final String             END_MESSAGE     = "CHORUS_EXEC_COUNT";

    private final ScheduledExecutorService threadPoolExecutor;

    public TimerangeExecutionFactory()
    {
        super();
        this.setSourceRequired(false);
        this.setSourceRequiredForMetadata(false);
        this.threadPoolExecutor = Executors.newScheduledThreadPool(1);
    }

    @Override
    public ResultSetExecution createResultSetExecution(final QueryExpression command,
                                                       final ExecutionContext executionContext,
                                                       final RuntimeMetadata metadata,
                                                       final Object connection) throws TranslatorException
    {
        return new TimerExecution(command,
                                  executionContext,
                                  this.threadPoolExecutor);
    }

    @Override
    public void getMetadata(final MetadataFactory metadataFactory,
                            final Object conn) throws TranslatorException
    {
        final Table timeRangeTable = metadataFactory.addTable("timetable");
        metadataFactory.addColumn(START,
                                  TypeFacility.RUNTIME_NAMES.TIMESTAMP,
                                  timeRangeTable)
                       .setSearchType(SearchType.Unsearchable);
        metadataFactory.addColumn(STOP,
                                  TypeFacility.RUNTIME_NAMES.TIMESTAMP,
                                  timeRangeTable)
                       .setSearchType(SearchType.Unsearchable);
        metadataFactory.addColumn(PERIOD,
                                  TypeFacility.RUNTIME_NAMES.LONG,
                                  timeRangeTable)
                       .setSearchType(SearchType.Searchable);
        metadataFactory.addColumn(ACTUAL_PERIOD,
                                  TypeFacility.RUNTIME_NAMES.LONG,
                                  timeRangeTable)
                       .setSearchType(SearchType.Unsearchable);
        metadataFactory.addColumn(EXECUTION_COUNT,
                                  TypeFacility.RUNTIME_NAMES.LONG,
                                  timeRangeTable)
                       .setSearchType(SearchType.Unsearchable);
        timeRangeTable.setCardinality(1);
        final Procedure exactlyOnceProcedure = metadataFactory.addProcedure("exactly_once");
        metadataFactory.addProcedureResultSetColumn("dummy",
                                                    TypeFacility.getDataTypeName(String.class),
                                                    exactlyOnceProcedure);
    }

    @Override
    public Void getConnection(final Object factory) throws TranslatorException
    {
        return null;
    }

    @Override
    public boolean supportsCompareCriteriaEquals()
    {
        return true;
    }

    @Override
    public boolean supportsRowLimit()
    {
        return true;
    }

    @Override
    public boolean isForkable()
    {
        return false;
    }

}
